class Wiktionary(object):

    def __init__(self, cfg):
        self.cfg = cfg
        self.init_parsers()
        self.pairs = list()

    def init_parsers(self):
        self.parsers = list()
        for parser_cl, parser_cfg in self.cfg.parsers:
            self.parsers.append(parser_cl(self.cfg, parser_cfg))

    def parse_articles(self, write_immediately=False):
        with open(self.cfg.output_path, 'w') as self.outf:
            for title, text in self.read_dump():
                pairs = self.extract_translations(title, text)
                if pairs:
                    if write_immediately:
                        self.write_one_article_translations(pairs)
                    else:
                        self.store_translations(pairs)
            if write_immediately is False:
                self.write_all_pairs()

    def extract_translations(self, title, text):
        if self.skip_article(title, text):
            return
        pairs = list()
        for parser in self.parsers:
            for p in parser.extract_translations(title, text):
                if len(p) == 2:
                    pair = ((self.cfg.wc, title, p[0], p[1]), tuple(parser.cfg.features))
                elif len(p) == 4:
                    pair = (p, tuple(parser.cfg.features))
                else:
                    raise Exception('Invalid pair {0}'.format(p))
                pairs.append(pair)
        return set(pairs)

    def skip_article(self, title, text):
        if not title.strip() or not text.strip():
            return True
        if ':' in title:  # skipping namespaced articles
            return True
        return False

    def write_one_article_translations(self, pairs):
        for pair in pairs:
            if self.cfg.verbose_output is True:
                self.outf.write('\t'.join(pair[0]) + '\n')
            else:
                self.outf.write('\t'.join(pair[0:4]) + '\n')

    def store_translations(self, pairs):
        for pair, feat in pairs:
            wc1, w1, wc2, w2 = pair[0:4]
            if wc1 < wc2:
                # storing source article too
                self.pairs.append([wc1, w1, wc2, w2, self.cfg.wc, w1] + list(feat))
            else:
                self.pairs.append([wc2, w2, wc1, w1, wc1, w1] + list(feat))

    def write_all_pairs(self):
        for pair in sorted(self.pairs):
            if self.cfg.verbose_output is True:
                self.outf.write('\t'.join(pair) + '\n')
            else:
                self.outf.write('\t'.join(pair[0:4]) + '\n')

    def read_dump(self):
        with open(self.cfg.dump_path) as f:
            title = ''
            article = ''
            page_sep = '%%#PAGE'
            for l_ in f:
                l = l_
                if l.startswith(page_sep):
                    if title and article:
                        yield title, article
                    title = l.split(page_sep)[-1].strip()
                    article = ''
                else:
                    article += l
            yield title, article



from sys import argv, stdin, stderr
from collections import defaultdict
import json


def read_table(fn):
    mapping = defaultdict(set)
    with open(fn) as f:
        for l in f:
            fd = l.decode('utf8').strip().split('\t')
            id_ = int(fd[0])
            for i, lang in enumerate(['en', 'hu', 'la', 'pl']):
                if fd[i + 1] == '#':
                    continue
                for word in fd[i + 1].split('/'):
                    mapping[(lang, word.replace('_', ' '))].add(id_)
    return mapping


def read_words(fn):
    words = set()
    with open(fn) as f:
        for l in f:
            fd = l.decode('utf8').strip().split('\t')
            if len(fd) >= 2:
                words.add((fd[0], fd[1]))
                if len(fd) >= 4:
                    words.add((fd[2], fd[3]))
    return words


def find_translations(words):
    iter_no = 0
    for l in stdin:
        iter_no += 1
        if iter_no % 1000000 == 0:
            stderr.write('{}\n'.format(iter_no))
        try:
            fd = l.decode('utf8').strip().split('\t')
            pair1 = (fd[0], fd[1])
            pair2 = (fd[2], fd[3])
            if pair1 in words:
                print(('\t'.join(fd[0:4] + list(pair1)).encode('utf8')))
            if pair2 in words:
                print(('\t'.join(fd[0:4] + list(pair2)).encode('utf8')))
        except ValueError:
            stderr.write('Error in line {}'.format(l))


def add_orig_bindings(mapping, translations):
    for (wc, word), ids in mapping.items():
        for id_ in ids:
            translations[id_][wc].add(word)


def find_translations_to_table(mapping):
    iter_no = 0
    translations = defaultdict(lambda: defaultdict(set))
    add_orig_bindings(mapping, translations)
    for l in stdin:
        iter_no += 1
        if iter_no % 1000000 == 0:
            stderr.write('{}\n'.format(iter_no))
        try:
            fd = l.decode('utf8').strip().split('\t')
            wc1, w1, wc2, w2 = fd[0:4]
            if wc1 == 'roa_rup' or wc2 == 'roa_rup':
                continue
            wc1 = 'zh' if wc1 == 'cmn' else wc1
            wc2 = 'zh' if wc2 == 'cmn' else wc2
            pair1 = (wc1, w1)
            pair2 = (wc2, w2)
            for id_ in mapping[pair1]:
                translations[id_][wc2].add(w2)
            for id_ in mapping[pair2]:
                translations[id_][wc1].add(w1)
        except ValueError:
            stderr.write('Error in line {}'.format(l))
    for id_, trans in translations.items():
        trans_to_dump = dict()
        for wc, words in trans.items():
            trans_to_dump[wc] = sorted(words)
        print(('{0}\t{1}'.format(id_, json.dumps(trans_to_dump))))
        #out_str = [str(id_)]
        #for wc, words in sorted(trans.iteritems()):
            #out_str.append(u'{0}:{1}'.format(wc, ','.join(sorted(words))))
        #print('\t'.join(out_str).encode('utf8'))


def main():
    mode = argv[2] if len(argv) > 2 else 'direct'
    if mode == 'direct':
        words = read_words(argv[1])
        find_translations(words)
    elif mode == 'collect':
        table = read_table(argv[1])
        find_translations_to_table(table)

if __name__ == '__main__':
    main()



from collections import defaultdict
from os import listdir
from sys import stdin, stderr
import logging
import wikt2dict.config as config


class Evaluator(object):

    def __init__(self):
        self.wikt = defaultdict(lambda: defaultdict(lambda:
                                                    defaultdict(lambda: defaultdict(int))))

        self.cfg = config.WiktionaryConfig()
        self.edge_gran = list(range(11)) + [20, 30]
        self.pivot_gran = self.edge_gran

        self.wikicodes = sorted(self.cfg.wikicodes)
        self.feat_order = []
        self.feat_order.extend(['left_' + wc for wc in self.wikicodes])
        self.feat_order.extend(['right_' + wc for wc in self.wikicodes])
        self.feat_order.extend(['pivot_' + str(i) for i in self.pivot_gran])
        self.feat_order.extend(['left_' + str(i) for i in self.edge_gran])
        self.feat_order.extend(['right_' + str(i) for i in self.edge_gran])
        self.feat_order.extend(['pivot_langs_' + str(i) for i in self.pivot_gran])
        self.feat_order.extend(['left_disjunct_' + str(i) for i in self.edge_gran])
        self.feat_order.extend(['right_disjunct_' + str(i) for i in self.edge_gran])
        self.feat_order.extend(['left_langs_' + str(i) for i in self.edge_gran])
        self.feat_order.extend(['right_langs_' + str(i) for i in self.edge_gran])
        for wc in self.wikicodes:
            self.feat_order.extend([wc + '_pivot_' + str(i) for i in self.pivot_gran])
        for wc in self.wikicodes:
            self.feat_order.extend([wc + '_left_' + str(i) for i in self.edge_gran])
        for wc in self.wikicodes:
            self.feat_order.extend([wc + '_right_' + str(i) for i in self.edge_gran])

    def write_labels(self, fn):
        f = open(fn, 'w')
        f.write('\n'.join('{0} {1}'.format(name, i) for i, name in enumerate(
            self.feat_order)))
        f.close()

    def read_all_wiktionary(self):
        for lang in listdir(self.cfg['dumpdir']):
            logging.info(lang)
            self.read_wiktionary(self.cfg['dumpdir'] + '/' + lang +
                                 '/' + self.cfg['word_pairs_outfile'])

    def read_wiktionary(self, fn):
        try:
            f = open(fn)
        except IOError:
            logging.info('{0} does not exist'.format(fn))
            return
        for l in f:
            try:
                l_ = l.decode('utf8').strip().split('\t')
                if len(l_) < 4:
                    logging.warning('Line too short: {0}'.format(l.strip()))
                    continue
                wc1, w1, wc2, w2 = self.get_ordered_pair(l_[0:4])

                if not wc1 in self.wikicodes or not wc2 in self.wikicodes:
                    continue
                self.wikt[wc1][w1][wc2][w2] += 1
            except:
                logging.exception('Exception at line: {0}'.format(l.strip()))

    def get_ordered_pair(self, fields):
        """ Order pairs alphabetically by Wiktionary codes.
        TODO this method should be moved to a static class containing
        often used methods """
        wc1, w1, wc2, w2 = fields
        if wc1 == 'cmn':
            wc1 = 'zh'
        if wc2 == 'cmn':
            wc2 = 'zh'
        if wc1 < wc2 or (wc1 == wc2 and w1 < w2):
            return wc1, w1, wc2, w2
        return wc2, w2, wc1, w1

    def compare_with_triangles_stdin(self):
        for l in stdin:
            try:
                l_ = l.decode('utf8').strip().split('\t')
                if len(l_) < 10:
                    logging.warning('Line too short: {0}'.format(l.strip()))
                    continue
                wc1, w1, wc2, w2 = self.get_ordered_pair(l_[0:4])
                if self.wikt[wc1][w1][wc2][w2] > 0:
                    print(l.strip() + '\t1')
                else:
                    print(l.strip() + '\t0')
            except:
                logging.exception('Exception at line: {0}'.format(l.strip()))

    def featurize_and_uniq_triangles_stdin(self):
        tri_group = set()
        tri_group_head = None
        cnt = 0
        for l in stdin:
            cnt += 1
            if cnt % 10000 == 0:
                stderr.write('{0}\n'.format(cnt))
            try:
                l_ = l.decode('utf8').strip().split('\t')
                this_tri = '\t'.join(l_[0:4])
                if not tri_group_head:
                    tri_group_head = this_tri
                    tri_group.add(tuple(l_))
                elif tri_group_head == this_tri:
                    tri_group.add(tuple(l_))
                else:
                    if tri_group:
                        feat, pair = self.featurize_group(tri_group)
                        self.print_pair_with_features(pair, feat)
                    tri_group = set()
                    tri_group.add(tuple(l_))
                    tri_group_head = this_tri
            except:
                logging.exception('Exception at line: {0}'.format(l.strip()))
        feat, pair = self.featurize_group(tri_group)
        self.print_pair_with_features(pair, feat)

    def print_pair_with_features(self, pair, feat):
        out = ''
        out += '\t'.join(pair) + '\t'
        for i, f in enumerate(self.feat_order):
            if f in feat and feat[f] == 1:
                out += '{0} '.format(i)
        print(out.strip().encode('utf8'))

    def featurize_group(self, group):
        feats = defaultdict(int)
        feats['total'] = len(group)
        left = set()
        right = set()
        pivots = set()
        pair = list()
        for i in group:
            left.add(tuple(i[4:6]))
            pivots.add(tuple(i[6:8]))
            right.add(tuple(i[8:10]))
            pair = i[0:4]
            if len(i) > 10:
                feats['in_wikt'] = int(i[10])
        self.map_int_feature_to_binaries('left', len(left), self.edge_gran, feats)
        self.map_int_feature_to_binaries('right', len(right), self.edge_gran, feats)
        self.map_int_feature_to_binaries('pivot', len(pivots), self.pivot_gran, feats)
        pivot_langs = len(set([lang for lang, _ in pivots]))
        self.map_int_feature_to_binaries('pivot_langs', pivot_langs,
                                         self.pivot_gran, feats)
        left_langs = set([lang for lang, _ in left])
        self.map_int_feature_to_binaries('left_langs', len(left_langs),
                                         self.edge_gran, feats)
        right_langs = set([lang for lang, _ in right])
        self.map_int_feature_to_binaries('right_langs', len(right_langs),
                                         self.edge_gran, feats)
        self.map_int_feature_to_binaries('left_disjunct', len(left_langs - right_langs), self.edge_gran, feats)
        self.map_int_feature_to_binaries('right_disjunct', len(right_langs - left_langs), self.edge_gran, feats)
        self.add_wc_features(pair, feats)

        self.map_int_feature_to_binaries_by_wc('_pivot', pivots,
                                               self.pivot_gran, feats)
        self.map_int_feature_to_binaries_by_wc('_left', left,
                                               self.edge_gran, feats)
        self.map_int_feature_to_binaries_by_wc('_right', right,
                                               self.edge_gran, feats)
        return feats, pair

    def map_int_feature_to_binaries_by_wc(self, pre, vals, granularity, feat_dict):
        val_wc = defaultdict(int)
        for wc, _ in vals:
            val_wc[wc] += 1
        for wc in self.wikicodes:
            self.map_int_feature_to_binaries(wc + pre, val_wc[wc],
                                             granularity, feat_dict)

    def add_wc_features(self, pair, feat_dict):
        for wc in self.wikicodes:
            if wc == pair[0]:
                feat_dict[wc + '_left'] = 1
            else:
                feat_dict[wc + '_left'] = 0
            if wc == pair[2]:
                feat_dict[wc + '_right'] = 1
            else:
                feat_dict[wc + '_right'] = 0

    def map_int_feature_to_binaries(self, feat_prefix, val, granularity, feat_dict,
                                    mode='le'):
        for i in granularity:
            feat_name = feat_prefix + '_' + str(i)
            if mode == 'le' and i <= val:
                feat_dict[feat_name] = 1
            elif mode == 'eq' and val == i:
                feat_dict[feat_name] = 1
            else:
                feat_dict[feat_name] = 0



#!/usr/bin/env python3
"""
Wikt2Dict

Usage:
  w2d.py (download|extract|triangulate|all) (--wikicodes=file|<wc>...)

Options:
  -h --help              Show this screen.
  --version              Show version.
  -w, --wikicodes=file   File containing a list of wikicodes.
"""

from docopt import docopt
from sys import stderr
from itertools import combinations
import subprocess, shlex
from urllib.request import urlretrieve
from os import path, getcwd
from subprocess import call
import logging
from wikt2dict.wiktionary import Wiktionary
from wikt2dict.triangulator import Triangulator
import wikt2dict.config as config


logging.basicConfig()
logger = logging.getLogger('wikt2dict')
logger.setLevel(logging.INFO)

def download_wiktionaries(wc_set):
    logger.info('Downloading Wiktionaries')
    to_download = [c for c in config.configs if c.wc in wc_set]
    for cfg in to_download:
        logger.info(cfg.dump_url)
        cmd = 'wget {0} -O {1}'.format(cfg.dump_url,cfg.bz2_path ) #It must have been changed according to the py >= 3.5 usage of the subprocess library
        wc_cmd = subprocess.run(shlex.split(cmd))
        wiki_textify_path = path.join(config.base_dir, "external/articles.py")
        call(['bzcat {0} | python {1} > {2}'.format(
            cfg.bz2_path, wiki_textify_path, cfg.dump_path)], shell=True) 

def extract_translations(wc_set):
    logger.info('Extracting translations')
    to_parse = [c for c in config.configs if c.wc in wc_set]
    for cfg in to_parse:
        #print(cfg.wc)
        wikt = Wiktionary(cfg)
        wikt.parse_articles()


def triangulate(wc_set):
    n = len(wc_set)
    num_of_tr = n * (n - 1) * (n - 2) / 6
    i = 1
    for triangle_wc in combinations(wc_set, 3):
        stderr.write(str(i) + '/' + str(num_of_tr) + repr(triangle_wc) + '\n')
        i += 1
        logger.info(' '.join(triangle_wc) + ' triangle')
        triangulator = Triangulator(triangle_wc)
        triangulator.collect_triangles()
        triangulator.write_triangles()


def main():
    arguments = docopt(__doc__, version='Wikt2dict 2.1')
    if arguments['--wikicodes']:
        with open(arguments['--wikicodes']) as f:
            wc_set = set([l.strip() for l in f])
    else:
        wc_set = set(arguments['<wc>'])
    if arguments['download'] or arguments['all']:
        download_wiktionaries(wc_set)
    if arguments['extract'] or arguments['all']:
        extract_translations(wc_set)
    if arguments['triangulate'] or arguments['all']:
        triangulate(wc_set)

if __name__ == '__main__':
    main()



import re
import logging
from collections import defaultdict

template_re = re.compile(r"\{\{[^\}]*\}\}", re.UNICODE)
default_translation_re = re.compile(
    r"\{\{(t[\u00d8|\-\+])\|([^}]+)\}\}", re.UNICODE)
global_features = ["sourcewc", "article", "has_article"]

# tester method
def uprint(str_):
    print(str_)

class ArticleParser(object):
    """ Base class for all article parsers. 
        This class should not be instantiated.
    """

    def __init__(self, wikt_cfg, parser_cfg, filter_langs=None):
        self.cfg = parser_cfg
        self.wikt_cfg = wikt_cfg
        self.pairs = list()
        self.titles = set()
        self.stats = defaultdict(list)
        self.build_skip_re()
        self.build_trim_re()
        if self.cfg['lower'] and self.cfg['lower'] == 1:
            self.lower_all = True
        else:
            self.lower_all = False

    def build_trim_re(self):
        if self.cfg['trim_re']:
            self.trim_re = re.compile(r'' + self.cfg['trim_re'], 
                                      re.UNICODE)

    def build_skip_re(self):
        if not self.cfg['skip_translation']:
            self.skip_translation_re = None
        else:
            self.skip_translation_re = re.compile(r'' + self.cfg['skip_translation'], re.UNICODE)
        if not self.cfg['skip_translation_line']:
            self.skip_translation_line_re = None
        else:
            self.skip_translation_line_re = re.compile(self.cfg['skip_translation_line'], re.UNICODE)

    def skip_translation_line(self, line):
        if 'PAGENAME' in line:
            return True
        if self.skip_translation_line_re and self.skip_translation_line_re.search(line):
            return True
        return False

    def parse_article(self, article, source_wc=None):
        if self.skip_article(article) == True:
            self.stats["skip_article"].append(article[0])
            return None
        title, text = article
        if self.lower_all:
            text = text.lower()
        self.titles.add(title)
        self.stats["ok"].append(title)
        t = self.get_pairs(text)
        if t:
            self.store_translations(title, t, source_wc)

    def get_pairs(self, text):
        return dict()

    def skip_article(self, article):
        if not article[0] or not article[1]:
            return True
        if not article[1].strip() or not article[0].strip():
            return True
        # ASSUMPTION: articles with a namespace contain no useful data
        if ':' in article[0]:
            return True
        return False

    def store_translations(self, this_word, translations, source_wc=None):
        for wc in list(translations.keys()):
            if len(translations[wc]) > 0:
                self.pairs.extend(
                        [[source_wc, this_word, wc, i, "sourcewc=" + self.wc, \
                      "article=" + this_word] 
                     for i in translations[wc]])

    def write_word_pairs_to_file(self, append=True):
        """ Write output to file
        One pair and its features are written to tab separated file
        """
        fn = self.cfg['dumpdir'] + '/' + self.cfg['fullname'] + '/' + self.cfg[\
                      'word_pairs_outfile']
        if append:
            outf = open(fn, 'a+')
        else:
            outf = open(fn, 'w')
        for p in self.pairs:
            out_str = self.generate_out_str(self.add_features_to_word_pair(p))
            if out_str:
                outf.write(out_str)
        outf.close()

    def generate_out_str(self, pair):
        if not pair:
            return None
        if len(pair) < 4:
            return None
        # alphabetic order
        if pair[0] < pair[2]:
            outstr = "\t".join(pair[0:4])
        else:
            outstr = "\t".join(pair[2:4] + pair[0:2])
        feat_d = dict()
        for feat in pair[4:]:
            fields = feat.split('=')
            if not fields[0] in global_features:
                self.log_handler.error('Feature not found {0}'.format(feat))
                continue
            if len(fields) > 1:
                feat_d[fields[0]] = fields[1]
            else:
                feat_d[fields[0]] = '1'
        for feat in global_features:
            if feat in feat_d:
                outstr += "\t" + feat_d[feat]
            else:
                outstr += "\t0"
        outstr += "\n"
        return outstr

    def add_features_to_word_pair(self, pair):
        """ Adding features to translation pairs
        """
        # article of the word exists
        if pair[3] in self.titles:
            pair.append("has_article")
        return pair

    def trim_translation(self, text):
        if self.cfg['trim_re']:
            text = self.trim_re.sub(r'\1\2', text)
        text = text.replace('[', '')
        text = text.replace(']', '')
        text = text.replace('{', '')
        text = text.replace('}', '')
        return text.strip()





"""
    AutoCorpus: automatically extracts clean natural language corpora from
    publicly available datasets.

    wiki.py: implements WikiParser - a fast XML parser that extracts
             markup and titles of articles from wikipedia xml snapshots.



    Copyright (C) 2011 Maciej Pacula


    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import sys
import os
import re
import xml.parsers.expat as sax

def err(msg):
  """ Prints a message to stderr, terminating it with a newline """
  sys.stderr.write(msg + "\n")

class Article:
  """ Stores the contents of a Wikipedia article """
  def __init__(self, title, markup, is_redirect):
    self.title = title
    self.markup = markup
    self.is_redirect = is_redirect


class WikiParser:
  """Parses the Wikipedia XML and extracts the relevant data,
     such as sentences and vocabulary"""

  
  def __init__(self, callback, ignore_redirects=True):
    self.callback = callback
    self.ignore_redirects = ignore_redirects
    self.buffer_size = 10*1024*1024 # 10MB

    # Articles whose titles start with "<type>:" will be ignored.
    self.ignoredArticleTypes = ["wikipedia", "category", "template"]

    
    # setup the SAX XML parser and its callbacks
    self.xml_parser = sax.ParserCreate()
    self.xml_parser.StartElementHandler  = lambda name, attrs: self.xml_start_element(name, attrs)
    self.xml_parser.EndElementHandler    = lambda name:        self.xml_end_element(name)
    self.xml_parser.CharacterDataHandler = lambda data:        self.xml_char_data(data)

    # parser state
    self.article = None         # name of the current article
    self.section = None         # name of the current section
    self.word    = None         # current word
    self.enclosing_tags = []    # all enclosing tags (most recent first)
    self.text    = []           # contents of the current text element, in the order
                                # they come from the SAX parser
                                # (note: this is faster than concatenating on the fly)
    self.article = None         # article currently being processed

    
  def process(self):
    while True:
      buf = sys.stdin.read(self.buffer_size)
      if buf == "":
        break

      self.xml_parser.Parse(buf)
      

  def xml_char_data(self, data):
    self.text.append(data)
    pass

  def xml_start_element(self, name, attrs):
    name = name.lower()
    self.enclosing_tags = [name] + self.enclosing_tags
    self.text = []

    if name == "page":
      self.article = Article(None, None, False)

  def xml_end_element(self, name):
    name = name.lower()
    contents = "".join(self.text)

    # dispatch based on the type of the node
    if name == "title":
      self.article.title = contents
    elif name == "redirect":
      self.article.is_redirect = True
    elif name == "text":
      self.article.markup = contents
    elif name == "page":
      if self.ignore_redirects and self.article.is_redirect:
        pass
      else:
        self.new_article(self.article)
        self.article = None

    # clean up state associated with the node    
    if len(self.enclosing_tags) > 0 and name == self.enclosing_tags[0]:
      self.enclosing_tags = self.enclosing_tags[1:]
    else:
      err("Mismatched closing tag: " + name)
    self.text = []

  def new_article(self, article):
    if ':' in  article.title:
      articleType = article.title.split(':')[0].lower()
      if articleType in self.ignoredArticleTypes:
        return

    self.callback(article)

  def get_enclosing_tag(self):
    return None if len(self.enclosing_tags) == 0 else self.enclosing_tags[0]
    
  def close(self):
    """Releases all resources associated with this class"""
    pass



from wikt2dict.article import ArticleParser


class SectionAndArticleParser(ArticleParser):
    """
    Class for parsing Wiktionaries that have translation tables
    in foreign articles too and section-level parsing is required.
    e.g. dewiktionary has a translation section in the article
    about the English word dog. Therefore, we need to recognize
    the language of the title word (dog) and then parse the
    translation table.
    """

    def __init__(self, wikt_cfg, parser_cfg, filter_langs=None):
        ArticleParser.__init__(self, wikt_cfg, parser_cfg, filter_langs)
        self.read_section_langmap()

    def read_section_langmap(self):
        """
        The language of a section is determined based on its header.
        The header may or may not use language names.
        If a language name map is specified, then each section header
        will be looked up in that map.
        Otherwise wikicodes are used.
        """
        self.section_langmap = dict()
        if self.cfg.section_langmap:
            f = open(self.cfg.section_langmap)
            for l in f:
                fields = l.strip().split('\t')
                for langname in fields[1:]:
                    self.section_langmap[langname] = fields[0]
                    self.section_langmap[langname.title()] = fields[0]
            f.close()
        else:
            self.section_langmap = dict([(wc, wc) for wc in self.wikt_cfg.wikicodes])

    def extract_translations(self, title, text):
        translations = list()
        for section_lang, section in self.get_sections(text):
            for parser in self.wikt_cfg.section_parsers:
                pairs = parser.extract_translations(title, section)
                for p in pairs:
                    if self.wikt_cfg.allow_synonyms is False and p[0] == section_lang:
                        continue
                    translations.extend([(section_lang, title, p[0], p[1])
                                         for p in pairs])
        return set(translations)

    def get_sections(self, text):
        section_titles_i = list()
        lines = text.split('\n')
        for i, line in enumerate(lines):
            m = self.cfg.section_re.search(line)
            if m:
                lang = m.group(self.cfg.section_langfield)
                section_titles_i.append((i, lang))
        if not section_titles_i:
            return
        for i, (ind, lang) in enumerate(section_titles_i[:-1]):
            if lang in self.section_langmap:
                yield self.section_langmap[lang], \
                    '\n'.join(lines[ind:section_titles_i[i + 1][0]])
        last_lang = section_titles_i[-1][1]
        if last_lang in self.section_langmap:
            yield self.section_langmap[last_lang], '\n'.join(lines[section_titles_i[-1][0]:])


class LangnamesArticleParser(ArticleParser):
    """
    Class for parsing Wiktionaries that use simple lists for translations
    instead of templates """

    def __init__(self, wikt_cfg, parser_cfg, filter_langs=None):
        ArticleParser.__init__(self, wikt_cfg, parser_cfg, filter_langs)
        self.read_langname_mapping()

    def read_langname_mapping(self):
        self.mapping = dict()
        if self.cfg.langnames:
            f = open(self.cfg.langnames)
            for l in f:
                fields = l.strip().split('\t')
                for langname in fields[1:]:
                    self.mapping[langname] = fields[0]
                    self.mapping[langname.title()] = fields[0]
                    self.mapping[langname.lower()] = fields[0]
            f.close()
        else:
            self.mapping = dict([(wc, wc) for wc in self.wikt_cfg.wikicodes])

    def extract_translations(self, title, text):
        translations = list()
        for tr in self.cfg.translation_line_re.finditer(text):
            if self.skip_translation_line(tr.group(0)):
                continue
            langname = tr.group(self.cfg.language_name_field).lower()
            if not langname in self.mapping:
                continue
            wc = self.mapping[langname]
            entities = self.get_entities(tr.group(self.cfg.translation_field))
            for entity in entities:
                entity_clear = self.trim_translation(entity)
                if entity_clear:
                    translations.append((wc, entity_clear))
        return set(translations)

    def trim_translation(self, word):
        return word.replace('\n', ' ').strip()

    def get_entities(self, trans_field):
        trimmed = self.cfg.bracket_re.sub('', trans_field)
        entities = list()
        for e in self.cfg.delimiter_re.split(trimmed):
            for m in self.cfg.translation_re.finditer(e):
                word = m.group(1)
                if self.skip_entity(word):
                    continue
                entities.append(word)
        return set(entities)

    def skip_entity(self, entity):
        if self.cfg.skip_translation_re.search(entity):
            return True
        if self.cfg.junk_re and self.cfg.junk_re.search(entity):
            return True
        return False


class DefaultArticleParser(ArticleParser):

    def extract_translations(self, title, text):
        translations = list()
        for tr in self.cfg.trad_re.finditer(text):
            wc = tr.group(self.cfg.wc_field)
            if not wc or not wc.strip() or not wc in self.wikt_cfg.wikicodes:
                continue
            word = tr.group(self.cfg.word_field)
            if not word or not word.strip():
                continue
            word = word.strip()
            if self.skip_word(word):
                continue
            translations.append((wc, word))
        return set(translations)

    def skip_word(self, word):
        if self.cfg.skip_translation_re and self.cfg.skip_translation_re.search(word):
            return True
        if '\n' in word:
            return True
        return False



from sys import argv, path
path.append('../src')

from wiktionary import Wiktionary
import logging

def main():
    if argv[2] == 'all' and len(argv) > 3:
        wc_f = open(argv[3])
        wikicodes = [wc.strip() for wc in wc_f]
    else:
        wikicodes = [wc.strip() for wc in argv[2:]]
    cfg_fn = argv[1]
    logger = logging.getLogger('wikt2dict')
    for wc in wikicodes:
        try:
            print('Parsing ' + wc + 'wiktionary')
            wiktionary = None #FIXME error handling workaround 
            wiktionary = Wiktionary(wc, cfg_fn)
            logger.info('%s Wiktionary object built', wiktionary.cfg['fullname'])
            wiktionary.parse_all_articles()
            logger.info('%s Wiktionary articles parsed', wiktionary.cfg['fullname'])
            print('   Extracted {0} pairs'.format(len(wiktionary.article_parser.pairs)))
            wiktionary.write_pairs()
            logger.info('%s Wiktionary translations written to file', wiktionary.cfg['fullname'])
        except NotImplementedError as e:
            if wiktionary and wiktionary.cfg and wiktionary.cfg['fullname']:
                logger.error('%s Wiktionary unrecognized parser type', wiktionary.cfg['fullname'])
            else:
                logger.error('%s Wiktionary unrecognized parser type', wc)
            continue
        except AttributeError as e:
            print(e)
            continue
        except Exception as e:
            print(wc, str(e))
            continue

if __name__ == '__main__':
    main()




from sys import argv, stdin
from collections import defaultdict
import json
import math


def main():
    stats = defaultdict(list)
    topN = int(argv[2])
    for l in stdin:
        fd = l.decode('utf8').split('\t')
        trans = json.loads(fd[1])
        for wc, words in trans.items():
            stats[wc].append(len(words))
    s = 0
    fourlang_len = 3478.0
    for wc, stat in sorted(iter(stats.items()), key=lambda x: -len(x[1]))[0:topN]:
        s += len(stat)
        print(('{} {}'.format(wc, len(stat) / fourlang_len)))
    E = s / float(topN)
    var = 0.0
    for wc, stat in sorted(iter(stats.items()), key=lambda x: -len(x[1]))[0:topN]:
        var += (len(stat) - E) ** 2
    print(E)
    print('var: ' + str(math.sqrt(var / float(topN)) / fourlang_len))
    print(s / float(topN))
    print(s / float(topN) / fourlang_len)


if __name__ == '__main__':
    main()



""" Wikt2Dict - Polygons

Usage:
    find_polygons.py (polygons|clicks) [--illustrate] [--wc-filter=<file>] [--k=<int>] [<input>...]

Options:
    --wc-filter=<file>      Filter input to wikicodes specified in file.
    -h --help               Show this screen.
    --k=<int>               Length of polygons [default: 4].
    --output=<file>         Output location.
    --illustrate            Use ASCII art in output.
"""
from docopt import docopt
from sys import stdin, stderr
from collections import defaultdict
#from copy import copy


def read_pairs(wc_filter=None, input_files=None, use_stdin=False):
    tri = defaultdict(set)
    if use_stdin:
        for l in stdin:
            add_pair(l, tri, wc_filter)
    elif input_files:
        for fn in input_files:
            with open(fn) as f:
                for l in f:
                    add_pair(l, tri, wc_filter)
    return tri


def add_pair(l, tri, wc_filter):
    try:
        wc1, w1, wc2, w2 = l.decode('utf8').strip().split('\t')[0:4]
        if wc_filter and (not wc1 in wc_filter or not wc2 in wc_filter):
            return
        tri[(wc1, w1)].add((wc2, w2))
        tri[(wc2, w2)].add((wc1, w1))
    except ValueError:
        stderr.write('Invalid line: {0}'.format(l))


def find_k_long_polygons(pairs, k):
    if k == 1:
        for word in list(pairs.keys()):
            yield [word]
    else:
        for polygon in find_k_long_polygons(pairs, k - 1):
            for word in pairs[polygon[-1]]:
                if not word in polygon[1:]:
                    yield polygon + [word]


def find_and_print_polygons(pairs, found=None, k=4, mode='polygons'):
    for polygon in find_k_long_polygons(pairs, k + 1):
        if polygon[0] == polygon[-1]:
            output(pairs, found=polygon, mode=mode)


def find_k_clicks(pairs, k):
    if k == 1:
        for word in list(pairs.keys()):
            yield [word]
    else:
        for click in find_k_clicks(pairs, k - 1):
            if len(click) > k - 1:
                continue
            for word in pairs[click[-1]]:
                if word in click:
                    continue
                bad = False
                for c in click[:-2]:
                    if not word in pairs[c]:
                        bad = True
                if not bad:
                    click.append(word)
                    yield click


def find_and_print_clicks(pairs, k=4):
    for click in find_k_clicks(pairs, k):
        output(pairs, found=sorted(click), mode='clicks')


def output(pairs, found, mode):
    edge_density, new_pairs = edge_density_and_new_pairs(pairs, found)
    if mode == 'clicks' and edge_density == 1.0:
        if arguments['--illustrate']:
            print((' --> '.join(', '.join([i, j]) for i, j in found).encode('utf8')))
        else:
            print(('\t'.join('\t'.join([i, j]) for i, j in found).encode('utf8')))
    elif mode == 'polygons':
        for pair in new_pairs:
            if arguments['--illustrate']:
                print(('\t'.join(pair[0]).encode('utf8') + '\t' +
                      '\t'.join(pair[1]).encode('utf8') + '\t' +
                      str(edge_density) + '\t' +
                      (' --> '.join(', '.join(i) for i in found)).encode('utf8')))
            else:
                print(('\t'.join(pair[0]).encode('utf8') + '\t' +
                      '\t'.join(pair[1]).encode('utf8') + '\t' +
                      str(edge_density) + '\t' +
                      ('\t'.join('\t'.join(i) for i in found)).encode('utf8')))


def edge_density_and_new_pairs(pairs, cycle):
    new_pairs = list()
    all_pairs = list()
    for i, e1 in enumerate(cycle):
        for e2 in cycle[i + 1:-1]:
            all_pairs.append(sorted([e1, e2]))
            if not e2 in pairs[e1] and not e1 in pairs[e2]:
                new_pairs.append(sorted([e1, e2]))
    n = len(all_pairs)
    return 1 - float(len(new_pairs)) / (n * (n - 1) / 2), new_pairs

arguments = docopt(__doc__, version="Wikt2dict/Polygons 0.1")


def main():
    if arguments['--wc-filter']:
        with open(arguments['--wc-filter']) as f:
            wc_filter = set([wc.strip() for wc in f])
    else:
        wc_filter = None
    k = int(arguments['--k'])
    if arguments['<input>']:
        pairs = read_pairs(wc_filter, input_files=arguments['<input>'])
    else:
        pairs = read_pairs(wc_filter, use_stdin=True)
    stderr.write('Pairs read\n')
    stderr.write('Number of pairs {0}\n'.format(
        sum(len(v) for v in list(pairs.values())) / 2))
    if arguments['polygons']:
        find_and_print_polygons(pairs, k=k, mode='polygons')
    else:
        #find_and_print_polygons(pairs, k=k, mode='clicks')
        find_and_print_clicks(pairs, k=k)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
    AutoCorpus: automatically extracts clean natural language corpora from
    publicly available datasets.

    articles.py: front-end to WikiParser. Reads wikipedia xml snapshots
                 from stdin and prints out the markup to stdout. Individual
                 articles are separated with the page feed character (C character \f).



    Copyright (C) 2011 Maciej Pacula


    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import sys
import os
import re
import codecs
import xml.parsers.expat as sax
from optparse import OptionParser
from wiki import *
import html.parser
import html


def fprint(x):
  print(x)

def save_article(htmlparser, article, directory):
  f = None
  try:
    filename = re.sub("(\s+)|/", "_", article.title.lower()) + ".txt"
    f = codecs.open(os.path.join(directory, filename), encoding='utf-8', mode='w')
    #f.write(htmlparser.unescape(article.markup))
    f.write(html.unescape(article.markup))
    f.write("\n")
    f.close()
  except Exception as e:
    sys.stderr.write("\nError extracting article: " + str(e) + "\n")
    if f != None:
      f.close()

def print_article(htmlparser, article):
  try:
    print("%%#PAGE " + str(article.title))#.encode("utf-8"))
    #print(str(htmlparser.unescape(article.markup)))  # .encode("utf-8"))
    print(str(html.unescape(article.markup)))#.encode("utf-8"))
    print("\n\n\f")
  except IOError:
    sys.exit(0) # broken stdout => broken pipe
  except Exception as e:
    sys.stderr.write("\nError extracting article: " + str(e) + "\n")
if __name__ == "__main__":
  try:
    parser = OptionParser(usage="usage: %s [-d output-directory] <stdin>" % sys.argv[0])
    parser.add_option("-d",
                      action="store", type="string", dest="directory",
                      help="directory where to store the articles")
    (options, args) = parser.parse_args()

    if len(args) > 0:
      print(parser.usage)
      exit(1)

    htmlparser = html.parser.HTMLParser()
    if options.directory != None:
      do_article = lambda a: save_article(htmlparser, a, options.directory)
    else:
      do_article = lambda a: print_article(htmlparser, a)

    parser = WikiParser(do_article)
    parser.process()
    parser.close()
  except KeyboardInterrupt:
    sys.stderr.write("\n\nCancelled. Partial results may have been generated.\n")
    exit(1)



import re
import string

punct = re.escape(''.join(set(string.punctuation) - set('.-,\'\"')))

alpha_re = re.compile(r'[' + punct + ']', re.UNICODE)

from sys import stdin, stderr, argv

cnt = 0
mode = argv[1]

for l in stdin:
    cnt += 1
    if cnt % 100000 == 0:
        stderr.write('{0}\n'.format(cnt))
    if mode == 'pos':
        if alpha_re.search(l.decode('utf8')):
            print(l.strip())
    elif mode == 'neg':
        if not alpha_re.search(l.decode('utf8')):
            print(l.strip())



"""
This file contains all configuration necessary for wikt2dict.
Each Wiktionary edition has a configuration class associated with it.
Since Wiktionary editions tend to have similarities, these classes are
subclasses of general configuration classes.
I know that modifying the configuration is more difficult than
in a plain text configuration file, and I do apologize for it,
but it's much easier to use a Python configuration and gives a whole
lot of new features.
"""
from wikt2dict.article_parsers import DefaultArticleParser, LangnamesArticleParser, SectionAndArticleParser
import re
from os import path, makedirs, getcwd

"""
Here are the default parameters for configuration base classes.
They can be and are overriden in many classes.
"""
base_dir = path.dirname(__file__)
wiktionary_defaults = {
    "wikicodes_file": path.join(base_dir, "../res/wikicodes"),
    #"dump_path_base": path.join(base_dir, "../dat/wiktionary"),
    "dump_path_base": path.join(getcwd(), "dat/wiktionary"),
    "dump_file_postfix": "wiktionary.txt",
    'output_file': "translation_pairs",
    'verbose_output': True,
    'triangle_threshold': 0,
    #'triangle_dir': path.join(base_dir, "../dat/triangle"),
    'triangle_dir': path.join(getcwd(), "dat/triangle"),
    'triangle_verbose': True,
    'only_new_triangles': False,
}

parser_defaults = {
    'blacklist': ['PAGENAME', r'^[\d\-]*$'],  # words that should not appear
    'placeholder': '',
    # allow same language pairs
    # issue with ltwiktionary: multiword articles have t+ templates
    # for the words of the article title
    'allow_synonyms': True,
}

default_parser_defaults = {
    'translation_prefix': r't[\u00f8\+\-]?',
    'wc_field': 1,
    'word_field': 2,
    'features': ['defaultparser'],
}

langname_parser_defaults = {
    'language_name_field': 1,
    'translation_field': 2,
    'translation_entity_delimiter': ",",
    'translation_re': re.compile(r'\[\[([^\[\]]+)\]\]'),
    'features': ['langnamesparser'],
    'junk_re': None,
}

section_level_parser_defaults = {
    'section_langfield': 1,
    'features': ['sectionlevel'],
}


class DictLikeClass(object):
    """
    All classes inherit this class so that configurations
    can be used as dictionaries.
    """
    def __getitem__(self, key):
        return self.__dict__.get(key, None)

    def __setitem__(self, key, value):
        self.__dict__[key] = value


class WiktionaryConfig(DictLikeClass):
    """
    This is the default WiktionaryConfig that each edition's
    configuration class inherits.
    """
    def __init__(self):
        for key, value in wiktionary_defaults.items():
            self[key] = value
        self._parsers = None
        self._parser_configs = None
        self._wikicodes = None

    @property
    def wikicodes(self):
        if not path.exists(self.wikicodes_file):
            makedirs(self.wikicodes_file)
        if not self._wikicodes:
            print(self.wikicodes_file)
            with open(self.wikicodes_file) as f:
                self._wikicodes = set([l.strip() for l in f])
        return self._wikicodes

    @property
    def dump_url(self):
        return "http://dumps.wikimedia.org/{0}wiktionary" \
               "/latest/{0}wiktionary-latest-pages-meta-current.xml.bz2".format(
                   self.wc)

    @property
    def bz2_path(self):
        #print("base_dir: ",base_dir)
        #print("dump_path_base: ",self.dump_path_base)
        path_dir = path.join(base_dir, self.dump_path_base,self.full_name)
        #path_dir = path.join(path_dir, self.full_name)
        #print("path_dir2: ",path_dir)
        if not path.exists(path_dir):
            #print("azt hiszi, nem létezik: ", path_dir)
            makedirs(path_dir)

        return path.join(path_dir, self.wc + ".bz2")

    @property
    def dump_filename(self):
        return self.wc + self.dump_file_postfix

    @property
    def dump_path(self):
        prefix = path.join(self.dump_path_base, self.full_name)
        if not path.exists(prefix):
            makedirs(prefix)
        return path.join(prefix, self.dump_filename)

    @property
    def output_path(self):
        prefix = path.join(self.dump_path_base, self.full_name)
        if not path.exists(prefix):
            makedirs(prefix)
        return path.join(self.dump_path_base, self.full_name, self.output_file)

    @property
    def parsers(self):
        """ The parsers need to be initialized exactly once. """
        if not self._parsers:
            self._parsers = list()
            if self._parser_configs:
                for parser_cl, parser_cfg_cl, parser_cfg in self._parser_configs:
                    self._parsers.append((parser_cl, parser_cfg_cl(parser_cfg)))
        return self._parsers


class ParserConfig(DictLikeClass):

    defaults = parser_defaults

    def __init__(self, wikt_cfg=None, parser_cfg=None):
        self._skip_trans_re = None
        for key, value in self.defaults.items():
            self[key] = value
        if parser_cfg:
            for key, value in parser_cfg.items():
                self[key] = value
        if wikt_cfg:
            for key, value in wikt_cfg.items():
                self[key] = value

    @property
    def skip_translation_re(self):
        if self._skip_trans_re is None:
            if self.placeholder:
                self.blacklist.append(self.placeholder)
            if self.blacklist:
                self._skip_trans_re = re.compile(
                    r'(' + '|'.join(self.blacklist) + ')',
                    re.UNICODE)
            else:
                self._skip_trans_re = ''
        return self._skip_trans_re

    def __setitem__(self, key, value):
        self.__dict__[key] = value

    def __getitem__(self, key):
        return self.__dict__.get(key, None)


class DefaultParserConfig(ParserConfig):

    def __init__(self, wikt_cfg=None, parser_cfg=None):
        self.defaults.update(default_parser_defaults)
        self._trad_re = None
        super(DefaultParserConfig, self).__init__(wikt_cfg, parser_cfg)

    @property
    def trad_re(self):
        if not self._trad_re:
            self._trad_re = re.compile(r'\{\{' + self.translation_prefix +
                                       r'\|([^}|]+)\|'  # wikicode
                                       r'([^}|]*)'  # word
                                       r'(?:\|([^}|]*))*\}\}')  # rest
        return self._trad_re


class LangnamesParserConfig(ParserConfig):

    def __init__(self, wikt_cfg=None, parser_cfg=None):
        self.defaults.update(langname_parser_defaults)
        self._bracket_re = None
        self._delimiter_re = None
        self._translation_line_re = None
        super(LangnamesParserConfig, self).__init__(wikt_cfg, parser_cfg)

    @property
    def bracket_re(self):
        if not self._bracket_re:
            self._bracket_re = re.compile(r'\([^)]*\)')
        return self._bracket_re

    @property
    def delimiter_re(self):
        if not self._delimiter_re:
            self._delimiter_re = re.compile(self.translation_entity_delimiter)
        return self._delimiter_re

    @property
    def translation_line_re(self):
        if not self._translation_line_re:
            self._translation_line_re = re.compile(self.translation_line)
        return self._translation_line_re


class SectionLevelParserConfig(ParserConfig):

    def __init__(self, wikt_cfg=None, parser_cfg=None):
        self.defaults.update(section_level_parser_defaults)
        super(SectionLevelParserConfig, self).__init__(wikt_cfg, parser_cfg)


class SectionLevelWiktionaryConfig(WiktionaryConfig):

    def __init__(self):
        self._section_parsers = None
        self._section_parser_configs = None
        super(SectionLevelWiktionaryConfig, self).__init__()

    @property
    def section_parsers(self):
        if not self._section_parsers:
            self._section_parsers = list()
            if self._section_parser_configs:
                for parser_cl, parser_cfg_cl, parser_cfg in self._section_parser_configs:
                    self._section_parsers.append((parser_cl(self, parser_cfg_cl(parser_cfg))))
        return self._section_parsers


class DefaultWiktionaryConfig(WiktionaryConfig):

    def __init__(self):
        super(DefaultWiktionaryConfig, self).__init__()
        try:
            cfg = self.default_cfg
        except AttributeError:
            cfg = {}
        self._parser_configs = [
            [DefaultArticleParser, DefaultParserConfig, cfg]
        ]


class LangnamesWiktionaryConfig(WiktionaryConfig):

    def __init__(self):
        super(LangnamesWiktionaryConfig, self).__init__()
        try:
            cfg = self.langnames_cfg
        except AttributeError:
            cfg = {}
        self._parser_configs = [
            [LangnamesArticleParser, LangnamesParserConfig, cfg]
        ]


class EstonianConfig(LangnamesWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Estonian'
        self.wc = 'et'
        self.langnames_cfg = {
            'langnames': path.join(base_dir, '../res/langnames/estonian'),
            'translation_line': r'\*\s*([^\s:]+)\s*:(.*)',
        }
        super(EstonianConfig, self).__init__()


class PolishConfig(LangnamesWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Polish'
        self.wc = 'pl'
        self.langnames_cfg = {
            'langnames': path.join(base_dir, '../res/langnames/polish'),
            'translation_line': r'\*\s*(.*):\s*((\([^\)]*)?\s*(.*))',
            'trim_re': r'<!--.*(?=-->)-->',
            'translation_entity_delimiter': r',\|;',
        }
        super(PolishConfig, self).__init__()


class RussianConfig(LangnamesWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Russian'
        self.wc = 'ru'
        self.langnames_cfg = {
            'langnames': False,
            'translation_line': r'\|([^\=]+)\=\s*(.+)',
            'translation_entity_delimiter': r',\|;',
        }
        super(RussianConfig, self).__init__()


class UkranianConfig(RussianConfig):

    def __init__(self):
        super(UkranianConfig, self).__init__()
        self.full_name = 'Ukrainian'
        self.wc = 'uk'


class SlovenianConfig(LangnamesWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Slovenian'
        self.wc = 'sl'
        self.langnames_cfg = {
            'langnames': path.join(base_dir, '../res/langnames/slovenian'),
            'translation_line': r'\*\s*([^\s:]+)\s*:(.*)',
            'translation_entity_delimiter': r',\|;',
        }
        super(SlovenianConfig, self).__init__()


class SomaliConfig(SlovenianConfig):

    def __init__(self):
        super(SomaliConfig, self).__init__()
        self.full_name = 'Somali'
        self.wc = 'so'
        self.langnames_cfg = {
            'translation_line': r'\*\s*([^\s:]+)\s*:(.*)',
            'translation_entity_delimiter': r',\|;',
        }


class MalagasyConfig(LangnamesWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Malagasy'
        self.wc = 'mg'
        self.langnames_cfg = {
            'langnames': path.join(base_dir, '../res/langnames/malagasy'),
            'translation_line': r'\#\s*([^\s:]+)\s*:(.*)',
        }
        super(MalagasyConfig, self).__init__()


class HebrewConfig(LangnamesWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Hebrew'
        self.wc = 'he'
        self.langnames_cfg = {
            'langnames': path.join(base_dir, '../res/langnames/hebrew'),
            'translation_line': r'{{\u05ea\|([^|}]+)\|([^}]+)}}',
            'translation_re': re.compile(r'([^|]+)')
        }
        super(HebrewConfig, self).__init__()


class IdoConfig(LangnamesWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Ido'
        self.wc = 'io'
        self.langnames_cfg = {
            'langnames': False,
            'translation_line': r'\*\s*\{{([^{}]+)}}:\s*([^\n]+)',
            'skip_translation': r'\*:\s*',
        }
        super(IdoConfig, self).__init__()


class ChineseConfig(LangnamesWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Chinese'
        self.wc = 'zh'
        self.langnames_cfg = {
            'langnames': False,
            'translation_line': r'\*\s*\{{([^}]+)}}:\s*(.+)',
        }
        super(ChineseConfig, self).__init__()


class ItalianConfig(LangnamesWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Italian'
        self.wc = 'it'
        self.langnames_cfg = {
            'langnames': False,
            'translation_line': r':\*\{{([^}]+)}}:\s*(.*)',
            'skip_translation_line':
            r'(\<\!\-\-\s*inserisci|inserisci.*traduzioni|altri\ template\ utili)',
            'skip_translation': r'(:\*:|<\!|^\?+$)',
        }
        super(ItalianConfig, self).__init__()


class VietnameseConfig(LangnamesWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Vietnamese'
        self.wc = 'vi'
        self.langnames_cfg = {
            'langnames': False,
            'translation_line': r':\s*\*\s*\{{([^}]+)}}:\s*(.*)',
        }
        super(VietnameseConfig, self).__init__()


class FinnishConfig(LangnamesWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Finnish'
        self.wc = 'fi'
        self.langnames_cfg = {
            'langnames': path.join(base_dir, '../res/langnames/finnish'),
            'translation_line': r'\*\{{0,2}([^}]+)\}{0,2}:\s*(.*)',
        }
        super(FinnishConfig, self).__init__()


class BasqueConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Basque'
        self.wc = 'eu'
        self.default_cfg = {
            'translation_prefix': r'itz',
        }
        super(BasqueConfig, self).__init__()


class BulgarianConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Bulgarian'
        self.wc = 'bg'
        self.default_cfg = {
            'translation_prefix': '\u043f',
        }
        super(BulgarianConfig, self).__init__()


class CatalanConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Catalan'
        self.wc = 'ca'
        self.default_cfg = {
            'translation_prefix': r'trad[\-\+]?',
            'placeholder': r'\?',
        }
        super(CatalanConfig, self).__init__()


class CroatianConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Croatian'
        self.wc = 'hr'
        self.default_cfg = {
            'translation_prefix': 'pr',
        }
        super(CroatianConfig, self).__init__()


class CzechConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Czech'
        self.wc = 'cs'
        self.default_cfg = {
            'translation_prefix': r'P',
        }
        super(CzechConfig, self).__init__()


class EnglishConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'English'
        self.wc = 'en'
        super(EnglishConfig, self).__init__()


class GermanConfig(SectionLevelWiktionaryConfig):

    def __init__(self):
        super(GermanConfig, self).__init__()
        self.full_name = 'German'
        self.wc = 'de'
        self.allow_synonyms = False
        default_cfg = {
            'trim_re': r'(.*)\#.*(.*)',
            'translation_prefix': r'[\xdc\xfc]',
        }
        self._section_parser_configs = [
            [DefaultArticleParser, DefaultParserConfig, default_cfg]
        ]
        section_cfg = {
            'parsers': [[DefaultArticleParser, DefaultParserConfig, default_cfg]],
            'section_langmap': path.join(base_dir, '../res/langnames/german'),
            'section_re': re.compile(r'==.*\(\{\{[Ss]prache\|(.+)\}\}\)\s*==')
        }
        self._parser_configs = [
            [SectionAndArticleParser, SectionLevelParserConfig, section_cfg]
        ]


class LithuanianConfig(SectionLevelWiktionaryConfig):

    def __init__(self):
        super(LithuanianConfig, self).__init__()
        self.full_name = 'Lithuanian'
        self.wc = 'lt'
        self.allow_synonyms = False
        default_cfg = {
            'trim_re': r'(.*)\#.*(.*)',
        }
        self._section_parser_configs = [
            [DefaultArticleParser, DefaultParserConfig, default_cfg]
        ]
        section_cfg = {
            'parsers': [[DefaultArticleParser, DefaultParserConfig, default_cfg]],
            'section_langmap': False,
            'section_re': re.compile(r'^==\s*\{\{(.+)v\}\}\s*==$')
        }
        self._parser_configs = [
            [SectionAndArticleParser, SectionLevelParserConfig, section_cfg]
        ]


class EsperantoConfig(WiktionaryConfig):

    def __init__(self):
        super(EsperantoConfig, self).__init__()
        self.full_name = 'Esperanto'
        self.wc = 'eo'
        langnames_cfg = {
            'translation_line': '\*\s*\{{([^}]+)}}:\s*(.+)',
            'langnames': False,
            'junk_re': re.compile(r'(:[^:]*:|\{\{|\}\}|xxx)')
        }
        self._parser_configs = [
            [LangnamesArticleParser, LangnamesParserConfig, langnames_cfg],
            [DefaultArticleParser, DefaultParserConfig, {}]
        ]


class FrenchConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'French'
        self.wc = 'fr'
        self.default_cfg = {
            'translation_prefix': r'trad[\-\+]?',
        }
        super(FrenchConfig, self).__init__()


class GalicianConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Galician'
        self.wc = 'gl'
        super(GalicianConfig, self).__init__()


class GeorgianConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Georgian'
        self.wc = 'ka'
        self.default_cfg = {
            'translation_prefix': '\u10d7x*',
        }
        super(GeorgianConfig, self).__init__()


class GreekConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Greek'
        self.wc = 'el'
        self.default_cfg = {
            'translation_prefix': '\u03c4',
            'placeholder': r'[xX\u03c7\u03a7]{3}',
        }
        super(GreekConfig, self).__init__()


class HungarianConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Hungarian'
        self.wc = 'hu'
        super(HungarianConfig, self).__init__()


class IcelandicConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Icelandic'
        self.wc = 'is'
        self.default_cfg = {
            'translation_prefix': r'\xfe\xfd\xf0ing',
        }
        super(IcelandicConfig, self).__init__()


class KurdishConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Kurdish'
        self.wc = 'ku'
        self.default_cfg = {
            'translation_prefix': r'(?:W\+?|trad)',
        }
        super(KurdishConfig, self).__init__()


class LatinConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Latin'
        self.wc = 'la'
        self.default_cfg = {
            'translation_prefix': r'x',
        }
        super(LatinConfig, self).__init__()


class LimburgishConfig(FrenchConfig):

    def __init__(self):
        super(LimburgishConfig, self).__init__()
        self.full_name = 'Limburgish'
        self.wc = 'li'


class NorwegianConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Norwegian'
        self.wc = 'no'
        self.default_cfg = {
            'translation_prefix': r'(?:o|overs|t[\u00f8\+\-]?)',
        }
        super(NorwegianConfig, self).__init__()


class OccitanConfig(FrenchConfig):

    def __init__(self):
        super(OccitanConfig, self).__init__()
        self.full_name = 'Occitan'
        self.wc = 'oc'


class PortugueseConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Portuguese'
        self.wc = 'pt'
        self.default_cfg = {
            'translation_prefix': r'(?:trad[\-\+]?|xlatio)',
        }
        super(PortugueseConfig, self).__init__()


class SerbianConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Serbian'
        self.wc = 'sr'
        self.default_cfg = {
            'translation_prefix': '\u041f',
            'word_field': 3,
            'placeholder': 'XXX',
        }
        super(SerbianConfig, self).__init__()


class SlovakConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Slovak'
        self.wc = 'sk'
        self.default_cfg = {
            'translation_prefix': 'P',
        }
        super(SlovakConfig, self).__init__()


class SpanishConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Spanish'
        self.wc = 'es'
        self.default_cfg = {
            'translation_prefix': r'(?:trad|t)[\u00f8\+\-]?',
            'placeholder': r'[\d\?\-\u2013,]',
        }
        super(SpanishConfig, self).__init__()


class SwahiliConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Swahili'
        self.wc = 'sw'
        super(SwahiliConfig, self).__init__()


class SwedishConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Swedish'
        self.wc = 'sv'
        self.default_cfg = {
            'translation_prefix': r'\u00f6.?',
        }
        super(SwedishConfig, self).__init__()


class TurkishConfig(DefaultWiktionaryConfig):

    def __init__(self):
        self.full_name = 'Turkish'
        self.wc = 'tr'
        self.default_cfg = {
            'translation_prefix': '\xe7eviri',
        }
        super(TurkishConfig, self).__init__()


class DanishConfig(SpanishConfig):

    def __init__(self):
        super(DanishConfig, self).__init__()
        self.full_name = 'Danish'
        self.wc = 'da'


class DutchConfig(FrenchConfig):

    def __init__(self):
        super(DutchConfig, self).__init__()
        self.full_name = 'Dutch'
        self.wc = 'nl'


class RomanianConfig(FrenchConfig):

    def __init__(self):
        super(RomanianConfig, self).__init__()
        self.full_name = 'Romanian'
        self.wc = 'ro'


class IndonesianConfig(FrenchConfig):

    def __init__(self):
        super(IndonesianConfig, self).__init__()
        self.full_name = 'Indonesian'
        self.wc = 'id'

class ArmenianConfig(DefaultWiktionaryConfig): # LangnamesWiktionaryConfig has been replaced with DefaultWiktionaryConfig

    def __init__(self):
        self.full_name = 'Armenian'
        self.wc = 'hy'
        self.langnames_cfg = {
            'langnames': False,
            'translation_line': r'\*\s*\{{([^}]+)|\s*(.+)}}',
        }
        super().__init__()

configs = [
    ArmenianConfig(),
    BasqueConfig(),
    BulgarianConfig(),
    CatalanConfig(),
    ChineseConfig(),
    CzechConfig(),
    CroatianConfig(),
    DanishConfig(),
    DutchConfig(),
    EnglishConfig(),
    EsperantoConfig(),
    EstonianConfig(),
    FinnishConfig(),
    FrenchConfig(),
    GalicianConfig(),
    GermanConfig(),
    GreekConfig(),
    GeorgianConfig(),
    HebrewConfig(),
    HungarianConfig(),
    IcelandicConfig(),
    IdoConfig(),
    ItalianConfig(),
    IndonesianConfig(),
    KurdishConfig(),
    LatinConfig(),
    LithuanianConfig(),
    LimburgishConfig(),
    MalagasyConfig(),
    NorwegianConfig(),
    OccitanConfig(),
    PolishConfig(),
    PortugueseConfig(),
    RomanianConfig(),
    RussianConfig(),
    SerbianConfig(),
    SlovakConfig(),
    SlovenianConfig(),
    SomaliConfig(),
    SpanishConfig(),
    SwahiliConfig(),
    SwedishConfig(),
    TurkishConfig(),
    UkranianConfig(),
    VietnameseConfig(),
]


def get_config_by_wc(wc):
    cfg_l = [x for x in configs if x.wc == wc]
    if len(cfg_l) > 1:
        # TODO raise exception
        pass
    return cfg_l[0]



from sys import argv, stderr, path
path.append('../src')

from itertools import combinations
import logging

from handlers import ConfigHandler, LogHandler
from triangulator import Triangulator

def main():
    if len(argv) > 2 and not argv[2] == 'all':
        filter_wc = set([wc.strip() for wc in argv[2:]])
    else:
        filter_wc = None
    cfg_fn = argv[1]
    logger = logging.getLogger('wikt2dict')
    cfg = ConfigHandler("general", cfg_fn)
    logger = LogHandler(cfg)
    with open(cfg['wikicodes']) as wc_f:
        wikicodes = set([w.strip() for w in wc_f])
    n = len(wikicodes)
    if filter_wc:
        m = n - len(filter_wc)
    else:
        m = 0
    num_of_tr = n * (n-1) * (n-2) / 6 - m * (m-1) * (m-2) / 6
    i = 1
    for triangle_wc in combinations(wikicodes, 3):
        if filter_wc and len(set(triangle_wc) & filter_wc) == 0:
            continue
        stderr.write(str(i) + '/' + str(num_of_tr) + repr(triangle_wc) + '\n')
        i += 1
        logger.info(' '.join(triangle_wc) + ' triangle')
        triangulator = Triangulator(triangle_wc, cfg_fn)
        triangulator.collect_triangles()
        triangulator.write_triangles()

if __name__ == '__main__':
    main()




from collections import defaultdict
from sys import stdin, stderr

def main():
    unigrams = defaultdict(lambda: defaultdict(int))
    for l in stdin:
        try:
            wc1, w1, wc2, w2 = l.decode('utf8').strip().split('\t')[0:4]
            for c in w1:
                unigrams[wc1][c] += 1
            for c in w2:
                unigrams[wc2][c] += 1
        except ValueError:
            stderr.write('Invalid line: {0}'.format(l))
    for wc, chars in unigrams.items():
        for c, cnt in sorted(((k, v) for k, v in chars.items()), key=lambda x: -x[1]):
            print(('{0}\t{1}\t{2}'.format(wc, c, cnt).encode('utf8')))

            
if __name__ == '__main__':
    main()



from os import path, listdir, makedirs
from collections import defaultdict
from itertools import product

import wikt2dict.config as config


class Triangulator(object):

    def __init__(self, triangle_wc):
        self.wikicodes = set(triangle_wc)
        self.cfg = config.WiktionaryConfig()
        self.pairs = defaultdict(lambda: defaultdict(
            lambda: defaultdict(lambda: defaultdict(list))))
        self.triangles = defaultdict(list)
        self.read_pairs_in_three_langs()

    def read_pairs_in_three_langs(self):
        for wc in self.wikicodes | set(['de', 'lt']):
            try:
                cfg = config.get_config_by_wc(wc)
                self.read_pairs_in_lang(wc, cfg.output_path)
            except IndexError:
                continue

    def read_pairs_in_lang(self, wc, fn):
        if not path.exists(fn):
            return
        with open(fn) as f:
            for l in f:
                fd = l.strip().split('\t') #.decode('utf8')
                if len(fd) < 6:
                    continue
                wc1, w1, wc2, w2, src_wc, src_art = fd[0:6]
                # converting Mandarin Chinese to Chinese
                if wc1 == 'cmn':
                    wc1 = 'zh'
                if wc2 == 'cmn':
                    wc2 = 'zh'
                if not wc1 in self.wikicodes and not wc2 in self.wikicodes:
                    continue
                self.pairs[wc1][w1][wc2][w2].append((src_wc, src_art))
                self.pairs[wc2][w2][wc1][w1].append((src_wc, src_art))

    def collect_triangles(self):
        for wc2 in self.wikicodes:  # this is the bridge language
            wc1, wc3 = sorted([w for w in self.wikicodes if not w == wc2])
            for w2, tr in self.pairs[wc2].items():
                for w1, src1_l in tr[wc1].items():
                    for w3, src3_l in tr[wc3].items():
                        for pair in product(src1_l, src3_l):
                            if wc1 < wc3:
                                self.triangles[(wc1, w1, wc3, w3)].append((
                                    pair[0][0], pair[0][1], wc2, w2, pair[1][0], pair[1][1]))
                            else:
                                self.triangles[(wc3, w3, wc1, w1)].append((
                                    pair[0][0], pair[0][1], wc2, w2, pair[1][0], pair[1][1]))

    def write_triangles(self):
        dir_ = self.get_dir()
        if not path.exists(dir_):
            makedirs(dir_)
        for wc2 in self.wikicodes:
            out_str = ''
            wc1, wc3 = sorted([w for w in self.wikicodes if not w == wc2])
            min_cnt = int(self.cfg.triangle_threshold)
            for tri, sources in self.triangles.items():
                if not tri[0] == wc1 or not tri[2] == wc3:
                    continue
                if len(sources) >= min_cnt:
                    for s in set(sources):
                        out_str += ('\t'.join(tri) + '\t' +
                                    '\t'.join(s) + '\n')    # '\t'.join(tri).encode('utf8'), '\t'.join(s).encode('utf8')
            if out_str:
                with open(dir_ + '/' + '_'.join([wc1, wc2, wc3]), 'w') as f:
                    f.write(out_str)

    def get_dir(self):
        i = 0
        file_cnt = 1000
        while file_cnt >= 998:
            dir_ = self.cfg['triangle_dir'] + '/' + str(i)
            i += 1
            if not path.exists(dir_):
                break
            file_cnt = len([name for name in listdir(dir_)])
        return dir_



from setuptools import setup, find_packages
from glob import glob

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()
import os
from setuptools import setup
from setuptools.command.develop import develop
from setuptools.command.install import install
from shutil import copyfile
from distutils.dir_util import copy_tree

HERE = os.path.abspath(os.path.dirname(__file__))
NAME = 'wikt2dict'

def copy_files (target_path):
    source_path = os.path.join(HERE)
    for fn in ["res"]:
        copy_tree(os.path.join(source_path, fn), os.path.join(target_path,fn))
    for fn in ["setup.py", "README.md"]:
        copyfile(os.path.join(source_path, fn), os.path.join(target_path,fn))

class PostDevelopCommand(develop):
    """Post-installation for development mode."""
    def run(self):
        develop.run(self)
        copy_files (os.path.abspath(NAME))

class PostInstallCommand(install):
    """Post-installation for installation mode."""
    def run(self):
        install.run(self)
        copy_files (os.path.abspath(os.path.join(self.install_lib)))

setup(
    author='Endre Hamerlik, Judit Acs',
    author_email='endre.hamerlik@fmph.uniba.sk, judit@sch.bme.hu',
    name=NAME,
    cmdclass={
        'develop': PostDevelopCommand,
        'install': PostInstallCommand,
    },
    version='2.1.0', 
    provides=['wikt2dict'],
    url='https://github.com/HMRLKE/wikt2dict',
    #packages=['wikt2dict'],
    packages=find_packages(),
    package_dir={'': '.'},
    #package_data={"res":["langnames/*", "*.py"]},
    #package_data={'': ['res/*.tsv']},
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=['docopt'],
    scripts=['wikt2dict/w2d.py'],
    classifiers=[
        'Programming Language :: Python :: 3.8',
    ],
    data_files=[
        ('res', glob('res/langnames/*')),
        ('res', glob('res/wik*')),
        ('bin', glob('bin/*.py')),
        ('', glob('*yml')),
        #('w2d_env', glob('w2d_env/*')),
        ('wikt2dict/external', glob('wikt2dict/external/*.py'))
    ],
    
    description="""Wiktionary translation parser tool for many language editions.
    Wikt2dict parses only the translation sections.
    It also has a triangulation mode which combines the extracted translation pairs to generate new ones.""",
    long_description=long_description,
    long_description_content_type='text/markdown'
)



""" Wikt2Dict - Find anomalies

Usage:
    find_anomalies.py punct [--num=N]
    find_anomalies.py unigram <unigram_file> [--whitelist=<whlist>...] [--] <prob_threshold>

Options:
    -n --num=N          Number of weird characters [default: 1].
    -h                  Show this screen.
"""
from sys import stdin, stderr
from docopt import docopt
from collections import defaultdict
import re
import string
import math

punct_ok = set('-\'.,?"')
punct = ''.join(set(string.punctuation) - punct_ok)
punct_re = re.compile(r'[{0}]'.format(re.escape(punct)),
                      re.UNICODE)

unigrams = defaultdict(lambda: defaultdict(int))
sum_ = defaultdict(int)


def scan_stdin(args):
    stats = {'punct': 0, 'punct ok': 0, 'sum': 0, 'invalid': 0}
    for l in stdin:
        stats['sum'] += 1
        try:
            wc1, w1, wc2, w2 = l.decode('utf8').strip().split('\t')[0:4]
            if args['punct']:
                if abs(len(punct_re.findall(w1)) - len(punct_re.findall(w2))) >= int(args['--num']):
                    print((l.strip()))
                    stats['punct'] += 1
                else:
                    stats['punct ok'] += 1
            if args['unigram']:
                if not wc1 in sum_ or not wc2 in sum_:
                    stderr.write('INVALID, unknown language: {0}'.format(l))
                    continue
                if wc1 in args['--whitelist'] or wc2 in args['--whitelist']:
                    continue
                prob1 = 0.0
                prob2 = 0.0
                for c in w1:
                    prob1 += math.log(float(unigrams[wc1][c]) / sum_[wc1])
                prob1 /= len(w1)
                for c in w2:
                    prob2 += math.log(float(unigrams[wc2][c]) / sum_[wc2])
                prob2 /= len(w2)
                if prob1 < int(args['<prob_threshold>']) or prob2 < int(args['<prob_threshold>']):
                    print((l.strip()))
        except ValueError:
            stats['invalid'] += 1
            stderr.write('INVALID: {0}'.format(l))


def read_unigrams(fn):
    with open(fn) as f:
        for l in f:
            wc, c, cnt = l.decode('utf8').split('\t')
            unigrams[wc][c] = int(cnt)
            sum_[wc] += int(cnt)


def main():
    args = docopt(__doc__, version="Wikt2Dict - Find anomalies 1.0")
    if args['unigram']:
        read_unigrams(args['<unigram_file>'])
    scan_stdin(args)

if __name__ == '__main__':
    main()



from sys import argv, stdin, stderr

def main():
    filt = set()
    mode = argv[2]
    with open(argv[1]) as f:
        for l in f:
            fd = l.decode('utf8').strip().lower().split('\t')
            if len(fd) < 4:
                stderr.write('Error on line {1}'. format(l))
                continue
            filt.add(tuple(fd[0:4]))
    stderr.write('Filter file read\n')
    cnt = 0
    for l in stdin:
        cnt += 1
        if cnt % 1000000 == 0:
            stderr.write('{0}\n'.format(cnt))
        fd = l.decode('utf8').strip().lower().split('\t')
        if len(fd) < 4:
            stderr.write('Error on line {1}'. format(l))
            continue
        if mode == 'pos' and tuple(fd[0:4]) in filt:
            print(l.strip())
        elif mode == 'neg' and not tuple(fd[0:4]) in filt:
            print(l.strip())


if __name__ == '__main__':
    main()



from sys import argv

from evaluator import Evaluator


def main():
    mode = argv[1]
    e = Evaluator()
    if mode == 'wikt':
        e.read_all_wiktionary()
        e.compare_with_triangles_stdin()
    elif mode == 'feat':
        e.write_labels(argv[2])
        e.featurize_and_uniq_triangles_stdin()

if __name__ == '__main__':
    main()
